"""TennisAbstract venue model."""

import datetime

import pytest_is_running
import requests_cache

from ....cache import MEMORY
from ...google.google_address_model import create_google_address_model
from ...venue_model import VERSION, VenueModel


def _create_tennisabstract_venue_model(
    grounds: str,
    session: requests_cache.CachedSession,
    dt: datetime.datetime,
    version: str,
) -> VenueModel:
    address_model = create_google_address_model(grounds, session, dt)
    return VenueModel(
        identifier=grounds,
        name=grounds,
        address=address_model,
        is_grass=None,
        is_indoor=None,
        is_turf=None,
        is_dirt=None,
        version=version,
    )


@MEMORY.cache(ignore=["session"])
def _cached_create_tennisabstract_venue_model(
    grounds: str,
    session: requests_cache.CachedSession,
    dt: datetime.datetime,
    version: str,
) -> VenueModel:
    return _create_tennisabstract_venue_model(
        grounds=grounds, session=session, dt=dt, version=version
    )


def create_tennisabstract_venue_model(
    grounds: str,
    session: requests_cache.CachedSession,
    dt: datetime.datetime,
) -> VenueModel:
    """Create a TennisAbstract venue model."""
    if not pytest_is_running.is_running():
        return _cached_create_tennisabstract_venue_model(
            grounds=grounds, session=session, dt=dt, version=VERSION
        )
    with session.cache_disabled():
        return _create_tennisabstract_venue_model(
            grounds=grounds, session=session, dt=dt, version=VERSION
        )



"""The prototype class for a coach."""

import datetime
from typing import Any, Literal

from pydantic import BaseModel, Field

from .field_type import FFILL_KEY, TYPE_KEY, FieldType
from .sex import (FEMALE_GENDERS, GENDER_DETECTOR, MALE_GENDERS,
                  UNCERTAIN_GENDERS, Sex)

COACH_IDENTIFIER_COLUMN: Literal["identifier"] = "identifier"
COACH_NAME_COLUMN: Literal["name"] = "name"
COACH_BIRTH_DATE_COLUMN: Literal["birth_date"] = "birth_date"
COACH_AGE_COLUMN: Literal["age"] = "age"
COACH_SEX_COLUMN: Literal["sex"] = "sex"
VERSION = "0.0.1"


def _guess_sex(data: dict[str, Any]) -> str | None:
    name = data[COACH_NAME_COLUMN]
    gender_tag = GENDER_DETECTOR.get_gender(name)
    if gender_tag in MALE_GENDERS:
        return str(Sex.MALE)
    if gender_tag in FEMALE_GENDERS:
        return str(Sex.FEMALE)
    if gender_tag in UNCERTAIN_GENDERS:
        return None
    return None


class CoachModel(BaseModel):
    """The serialisable coach class."""

    identifier: str = Field(
        ...,
        json_schema_extra={TYPE_KEY: FieldType.CATEGORICAL},
        alias=COACH_IDENTIFIER_COLUMN,
    )
    name: str = Field(..., alias=COACH_NAME_COLUMN)
    birth_date: datetime.date | None = Field(
        ..., json_schema_extra={FFILL_KEY: True}, alias=COACH_BIRTH_DATE_COLUMN
    )
    age: int | None = Field(..., alias=COACH_AGE_COLUMN)
    version: str = Field(..., json_schema_extra={TYPE_KEY: FieldType.CATEGORICAL})
    sex: str | None = Field(
        default_factory=_guess_sex,
        json_schema_extra={TYPE_KEY: FieldType.CATEGORICAL, FFILL_KEY: True},
        alias=COACH_SEX_COLUMN,
    )



"""The prototype class for an umpire."""

# pylint: disable=duplicate-code
import datetime
from typing import Any, Literal

from pydantic import BaseModel, Field

from .address_model import VERSION as ADDRESS_VERSION
from .address_model import AddressModel
from .delimiter import DELIMITER
from .field_type import FFILL_KEY, TYPE_KEY, FieldType
from .sex import (FEMALE_GENDERS, GENDER_DETECTOR, MALE_GENDERS,
                  UNCERTAIN_GENDERS, Sex)
from .venue_model import VERSION as VENUE_VERSION
from .venue_model import VenueModel

UMPIRE_IDENTIFIER_COLUMN: Literal["identifier"] = "identifier"
UMPIRE_NAME_COLUMN: Literal["name"] = "name"
UMPIRE_SEX_COLUMN: Literal["sex"] = "sex"
UMPIRE_BIRTH_DATE_COLUMN: Literal["birth_date"] = "birth_date"
UMPIRE_AGE_COLUMN: Literal["age"] = "age"
UMPIRE_BIRTH_ADDRESS_COLUMN: Literal["birth_address"] = "birth_address"
UMPIRE_HIGH_SCHOOL_COLUMN: Literal["high_school"] = "high_school"
VERSION = DELIMITER.join(["0.0.1", ADDRESS_VERSION, VENUE_VERSION])


def _guess_sex(data: dict[str, Any]) -> str | None:
    name = data[UMPIRE_NAME_COLUMN]
    gender_tag = GENDER_DETECTOR.get_gender(name)
    if gender_tag in MALE_GENDERS:
        return str(Sex.MALE)
    if gender_tag in FEMALE_GENDERS:
        return str(Sex.FEMALE)
    if gender_tag in UNCERTAIN_GENDERS:
        return None
    return None


class UmpireModel(BaseModel):
    """The serialisable umpire class."""

    identifier: str = Field(
        ...,
        json_schema_extra={TYPE_KEY: FieldType.CATEGORICAL},
        alias=UMPIRE_IDENTIFIER_COLUMN,
    )
    name: str = Field(..., alias=UMPIRE_NAME_COLUMN)
    sex: str | None = Field(
        default_factory=_guess_sex,
        json_schema_extra={TYPE_KEY: FieldType.CATEGORICAL, FFILL_KEY: True},
        alias=UMPIRE_SEX_COLUMN,
    )
    birth_date: datetime.date | None = Field(
        ..., json_schema_extra={FFILL_KEY: True}, alias=UMPIRE_BIRTH_DATE_COLUMN
    )
    age: int | None = Field(..., alias=UMPIRE_AGE_COLUMN)
    birth_address: AddressModel | None = Field(
        ..., json_schema_extra={FFILL_KEY: True}, alias=UMPIRE_BIRTH_ADDRESS_COLUMN
    )
    high_school: VenueModel | None = Field(
        ..., json_schema_extra={FFILL_KEY: True}, alias=UMPIRE_HIGH_SCHOOL_COLUMN
    )
    version: str = Field(..., json_schema_extra={TYPE_KEY: FieldType.CATEGORICAL})



"""TennisAbstract team model."""

# pylint: disable=duplicate-code,too-many-locals,too-many-arguments
import datetime

import pytest_is_running
from scrapesession.scrapesession import ScrapeSession  # type: ignore

from ....cache import MEMORY
from ...delimiter import DELIMITER
from ...google.google_news_model import create_google_news_models
from ...league import League
from ...team_model import VERSION, TeamModel
from ...x.x_social_model import create_x_social_model
from .atp_tennisabstract_player_model import create_tennisabstract_player_model


def _create_tennisabstract_team_model(
    session: ScrapeSession,
    dt: datetime.datetime,
    league: League,
    player_urls: list[str],
    ace_percentages: list[float],
    double_fault_percentages: list[float],
    first_serves_ins: list[float],
    first_serve_percentages: list[float],
    second_serve_percentages: list[float],
    break_points_saveds: list[float],
    return_points_won_percentages: list[float],
    winners: list[int],
    winners_fronthands: list[int],
    winners_backhands: list[int],
    unforced_errors: list[int],
    unforced_errors_fronthand: list[int],
    unforced_errors_backhand: list[int],
    serve_points: list[int],
    serves_won: list[float],
    serves_aces: list[float],
    serves_unreturned: list[float],
    serves_forced_error_percentage: list[float],
    serves_won_in_three_shots_or_less: list[float],
    serves_wide_percentage: list[float],
    serves_body_percentage: list[float],
    serves_t_percentage: list[float],
    serves_wide_deuce_percentage: list[float],
    serves_body_deuce_percentage: list[float],
    serves_t_deuce_percentage: list[float],
    serves_wide_ad_percentage: list[float],
    serves_body_ad_percentage: list[float],
    serves_t_ad_percentage: list[float],
    serves_net_percentage: list[float],
    serves_wide_direction_percentage: list[float],
    shots_deep_percentage: list[float],
    shots_deep_wide_percentage: list[float],
    shots_foot_errors_percentage: list[float],
    shots_unknown_percentage: list[float],
    points_won_percentage: list[float],
    version: str,
    points: float,
) -> TeamModel:
    players = [
        create_tennisabstract_player_model(
            session=session,
            dt=dt,
            league=league,
            url=x,
            ace_percentages=ace_percentages,
            team_idx=count,
            double_fault_percentages=double_fault_percentages,
            first_serves_ins=first_serves_ins,
            first_serve_percentages=first_serve_percentages,
            second_serve_percentages=second_serve_percentages,
            break_points_saveds=break_points_saveds,
            return_points_won_percentages=return_points_won_percentages,
            winners=winners,
            winners_fronthands=winners_fronthands,
            winners_backhands=winners_backhands,
            unforced_errors=unforced_errors,
            unforced_errors_fronthand=unforced_errors_fronthand,
            unforced_errors_backhand=unforced_errors_backhand,
            serve_points=serve_points,
            serves_won=serves_won,
            serves_aces=serves_aces,
            serves_unreturned=serves_unreturned,
            serves_forced_error_percentage=serves_forced_error_percentage,
            serves_won_in_three_shots_or_less=serves_won_in_three_shots_or_less,
            serves_wide_percentage=serves_wide_percentage,
            serves_body_percentage=serves_body_percentage,
            serves_t_percentage=serves_t_percentage,
            serves_wide_deuce_percentage=serves_wide_deuce_percentage,
            serves_body_deuce_percentage=serves_body_deuce_percentage,
            serves_t_deuce_percentage=serves_t_deuce_percentage,
            serves_wide_ad_percentage=serves_wide_ad_percentage,
            serves_body_ad_percentage=serves_body_ad_percentage,
            serves_t_ad_percentage=serves_t_ad_percentage,
            serves_net_percentage=serves_net_percentage,
            serves_wide_direction_percentage=serves_wide_direction_percentage,
            shots_deep_percentage=shots_deep_percentage,
            shots_deep_wide_percentage=shots_deep_wide_percentage,
            shots_foot_errors_percentage=shots_foot_errors_percentage,
            shots_unknown_percentage=shots_unknown_percentage,
            points_won_percentage=points_won_percentage,
        )
        for count, x in enumerate(player_urls)
    ]
    identifier = DELIMITER.join([x.identifier for x in players])
    name = DELIMITER.join([x.name for x in players])
    return TeamModel(
        identifier=identifier,
        name=name,
        players=players,
        odds=[],
        points=points,
        ladder_rank=None,
        location=None,
        news=create_google_news_models(name, session, dt, league),
        social=create_x_social_model(name, session, dt),
        coaches=[],
        lbw=None,
        end_dt=None,
        runs=None,
        wickets=None,
        overs=None,
        balls=None,
        byes=None,
        leg_byes=None,
        wides=None,
        no_balls=None,
        penalties=None,
        balls_per_over=None,
        fours=None,
        sixes=None,
        catches=None,
        catches_dropped=None,
        version=version,
    )


@MEMORY.cache(ignore=["session"])
def _cached_create_tennisabstract_team_model(
    session: ScrapeSession,
    dt: datetime.datetime,
    league: League,
    player_urls: list[str],
    ace_percentages: list[float],
    double_fault_percentages: list[float],
    first_serves_ins: list[float],
    first_serve_percentages: list[float],
    second_serve_percentages: list[float],
    break_points_saveds: list[float],
    return_points_won_percentages: list[float],
    winners: list[int],
    winners_fronthands: list[int],
    winners_backhands: list[int],
    unforced_errors: list[int],
    unforced_errors_fronthand: list[int],
    unforced_errors_backhand: list[int],
    serve_points: list[int],
    serves_won: list[float],
    serves_aces: list[float],
    serves_unreturned: list[float],
    serves_forced_error_percentage: list[float],
    serves_won_in_three_shots_or_less: list[float],
    serves_wide_percentage: list[float],
    serves_body_percentage: list[float],
    serves_t_percentage: list[float],
    serves_wide_deuce_percentage: list[float],
    serves_body_deuce_percentage: list[float],
    serves_t_deuce_percentage: list[float],
    serves_wide_ad_percentage: list[float],
    serves_body_ad_percentage: list[float],
    serves_t_ad_percentage: list[float],
    serves_net_percentage: list[float],
    serves_wide_direction_percentage: list[float],
    shots_deep_percentage: list[float],
    shots_deep_wide_percentage: list[float],
    shots_foot_errors_percentage: list[float],
    shots_unknown_percentage: list[float],
    points_won_percentage: list[float],
    points: float,
    version: str,
) -> TeamModel:
    return _create_tennisabstract_team_model(
        session=session,
        dt=dt,
        league=league,
        player_urls=player_urls,
        ace_percentages=ace_percentages,
        double_fault_percentages=double_fault_percentages,
        first_serves_ins=first_serves_ins,
        first_serve_percentages=first_serve_percentages,
        second_serve_percentages=second_serve_percentages,
        break_points_saveds=break_points_saveds,
        return_points_won_percentages=return_points_won_percentages,
        winners=winners,
        winners_fronthands=winners_fronthands,
        winners_backhands=winners_backhands,
        unforced_errors=unforced_errors,
        unforced_errors_fronthand=unforced_errors_fronthand,
        unforced_errors_backhand=unforced_errors_backhand,
        serve_points=serve_points,
        serves_won=serves_won,
        serves_aces=serves_aces,
        serves_unreturned=serves_unreturned,
        serves_forced_error_percentage=serves_forced_error_percentage,
        serves_won_in_three_shots_or_less=serves_won_in_three_shots_or_less,
        serves_wide_percentage=serves_wide_percentage,
        serves_body_percentage=serves_body_percentage,
        serves_t_percentage=serves_t_percentage,
        serves_wide_deuce_percentage=serves_wide_deuce_percentage,
        serves_body_deuce_percentage=serves_body_deuce_percentage,
        serves_t_deuce_percentage=serves_t_deuce_percentage,
        serves_wide_ad_percentage=serves_wide_ad_percentage,
        serves_body_ad_percentage=serves_body_ad_percentage,
        serves_t_ad_percentage=serves_t_ad_percentage,
        serves_net_percentage=serves_net_percentage,
        serves_wide_direction_percentage=serves_wide_direction_percentage,
        shots_deep_percentage=shots_deep_percentage,
        shots_deep_wide_percentage=shots_deep_wide_percentage,
        shots_foot_errors_percentage=shots_foot_errors_percentage,
        shots_unknown_percentage=shots_unknown_percentage,
        points_won_percentage=points_won_percentage,
        points=points,
        version=version,
    )


def create_tennisabstract_team_model(
    session: ScrapeSession,
    dt: datetime.datetime,
    league: League,
    player_urls: list[str],
    ace_percentages: list[float],
    double_fault_percentages: list[float],
    first_serves_ins: list[float],
    first_serve_percentages: list[float],
    second_serve_percentages: list[float],
    break_points_saveds: list[float],
    return_points_won_percentages: list[float],
    winners: list[int],
    winners_fronthands: list[int],
    winners_backhands: list[int],
    unforced_errors: list[int],
    unforced_errors_fronthand: list[int],
    unforced_errors_backhand: list[int],
    serve_points: list[int],
    serves_won: list[float],
    serves_aces: list[float],
    serves_unreturned: list[float],
    serves_forced_error_percentage: list[float],
    serves_won_in_three_shots_or_less: list[float],
    serves_wide_percentage: list[float],
    serves_body_percentage: list[float],
    serves_t_percentage: list[float],
    serves_wide_deuce_percentage: list[float],
    serves_body_deuce_percentage: list[float],
    serves_t_deuce_percentage: list[float],
    serves_wide_ad_percentage: list[float],
    serves_body_ad_percentage: list[float],
    serves_t_ad_percentage: list[float],
    serves_net_percentage: list[float],
    serves_wide_direction_percentage: list[float],
    shots_deep_percentage: list[float],
    shots_deep_wide_percentage: list[float],
    shots_foot_errors_percentage: list[float],
    shots_unknown_percentage: list[float],
    points_won_percentage: list[float],
    points: float,
) -> TeamModel:
    """Create a team model from TennisAbstract."""
    if not pytest_is_running.is_running():
        return _cached_create_tennisabstract_team_model(
            session=session,
            dt=dt,
            league=league,
            player_urls=player_urls,
            ace_percentages=ace_percentages,
            double_fault_percentages=double_fault_percentages,
            first_serves_ins=first_serves_ins,
            first_serve_percentages=first_serve_percentages,
            second_serve_percentages=second_serve_percentages,
            break_points_saveds=break_points_saveds,
            return_points_won_percentages=return_points_won_percentages,
            winners=winners,
            winners_fronthands=winners_fronthands,
            winners_backhands=winners_backhands,
            unforced_errors=unforced_errors,
            unforced_errors_fronthand=unforced_errors_fronthand,
            unforced_errors_backhand=unforced_errors_backhand,
            serve_points=serve_points,
            serves_won=serves_won,
            serves_aces=serves_aces,
            serves_unreturned=serves_unreturned,
            serves_forced_error_percentage=serves_forced_error_percentage,
            serves_won_in_three_shots_or_less=serves_won_in_three_shots_or_less,
            serves_wide_percentage=serves_wide_percentage,
            serves_body_percentage=serves_body_percentage,
            serves_t_percentage=serves_t_percentage,
            serves_wide_deuce_percentage=serves_wide_deuce_percentage,
            serves_body_deuce_percentage=serves_body_deuce_percentage,
            serves_t_deuce_percentage=serves_t_deuce_percentage,
            serves_wide_ad_percentage=serves_wide_ad_percentage,
            serves_body_ad_percentage=serves_body_ad_percentage,
            serves_t_ad_percentage=serves_t_ad_percentage,
            serves_net_percentage=serves_net_percentage,
            serves_wide_direction_percentage=serves_wide_direction_percentage,
            shots_deep_percentage=shots_deep_percentage,
            shots_deep_wide_percentage=shots_deep_wide_percentage,
            shots_foot_errors_percentage=shots_foot_errors_percentage,
            shots_unknown_percentage=shots_unknown_percentage,
            points_won_percentage=points_won_percentage,
            points=points,
            version=VERSION,
        )
    with session.cache_disabled():
        return _create_tennisabstract_team_model(
            session=session,
            dt=dt,
            league=league,
            player_urls=player_urls,
            ace_percentages=ace_percentages,
            double_fault_percentages=double_fault_percentages,
            first_serves_ins=first_serves_ins,
            first_serve_percentages=first_serve_percentages,
            second_serve_percentages=second_serve_percentages,
            break_points_saveds=break_points_saveds,
            return_points_won_percentages=return_points_won_percentages,
            winners=winners,
            winners_fronthands=winners_fronthands,
            winners_backhands=winners_backhands,
            unforced_errors=unforced_errors,
            unforced_errors_fronthand=unforced_errors_fronthand,
            unforced_errors_backhand=unforced_errors_backhand,
            serve_points=serve_points,
            serves_won=serves_won,
            serves_aces=serves_aces,
            serves_unreturned=serves_unreturned,
            serves_forced_error_percentage=serves_forced_error_percentage,
            serves_won_in_three_shots_or_less=serves_won_in_three_shots_or_less,
            serves_wide_percentage=serves_wide_percentage,
            serves_body_percentage=serves_body_percentage,
            serves_t_percentage=serves_t_percentage,
            serves_wide_deuce_percentage=serves_wide_deuce_percentage,
            serves_body_deuce_percentage=serves_body_deuce_percentage,
            serves_t_deuce_percentage=serves_t_deuce_percentage,
            serves_wide_ad_percentage=serves_wide_ad_percentage,
            serves_body_ad_percentage=serves_body_ad_percentage,
            serves_t_ad_percentage=serves_t_ad_percentage,
            serves_net_percentage=serves_net_percentage,
            serves_wide_direction_percentage=serves_wide_direction_percentage,
            shots_deep_percentage=shots_deep_percentage,
            shots_deep_wide_percentage=shots_deep_wide_percentage,
            shots_foot_errors_percentage=shots_foot_errors_percentage,
            shots_unknown_percentage=shots_unknown_percentage,
            points_won_percentage=points_won_percentage,
            points=points,
            version=VERSION,
        )
